# -*- coding: utf-8 -*-
"""cdns plugin"""

__author__  = "Adrien DELLE CAVE"
__license__ = """
    Copyright (C) 2018  fjord-technologies

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA..
"""

import cdnetworks
import json
import logging
import os

from dwho.adapters.redis import DWhoAdapterRedis
from dwho.classes.plugins import PLUGINS
from nsaproxy.classes.apis import NSAProxyApiBase, NSAProxyApiSync, APIS_SYNC
from sonicprobe import helpers


LOG = logging.getLogger('nsaproxy.plugins.cdns')

#logging.getLogger('requests').setLevel(logging.WARNING)


class NSAProxyCdnsPlugin(NSAProxyApiBase):
    PLUGIN_NAME = 'cdns'

    def safe_init(self):
        self.conn           = None

        if self.PLUGIN_NAME not in self.config['plugins']:
            return

        self.adapter_redis  = DWhoAdapterRedis(self.config, prefix = 'nsaproxy')

        if self.config['plugins'][self.PLUGIN_NAME].get('credentials'):
            cred = helpers.load_yaml_file(self.config['plugins'][self.PLUGIN_NAME]['credentials'])
            if not cred:
                raise ValueError("unable to read credentials")

            for k, v in cred['cdnetworks'].iteritems():
                os.environ[k.upper()] = v

        self.conn = cdnetworks.service('cdns')

        APIS_SYNC.register(NSAProxyApiSync(self.PLUGIN_NAME))

    def at_start(self):
        if self.PLUGIN_NAME in APIS_SYNC:
            self.start()

    def _is_in_cache(self, rrcache, change):
        for record in rrcache:
            if cmp(record, change) == 0:
                return True

        return False

    def _do_create_hosted_zone(self, obj):
        args = obj.get_args()
        self.adapter_redis.set_key("nsa.%s.hzone:%s"
                                   % (self.PLUGIN_NAME, args['name'].rstrip('.')), '')

    def _do_delete_hosted_zone(self, obj):
        return

    def _do_change_rrsets(self, obj):
        params  = obj.get_params()

        hostid  = params['id'].rstrip('.')
        xid     = self.adapter_redis.get_key("nsa.%s.hzone:%s"
                                             % (self.PLUGIN_NAME, hostid))
        if not xid:
            domains = self.conn.search_domains(hostid)
            if not domains or 'domains' not in domains:
                raise LookupError("unable to find zone id: %r" % hostid)
            self.adapter_redis.set_key("nsa.%s.hzone:%s" % (self.PLUGIN_NAME, hostid),
                                       domains['domains']['domains'][0]['domain_id'])

        xid     = self.adapter_redis.get_key("nsa.%s.hzone:%s"
                                             % (self.PLUGIN_NAME, hostid))
        if not xid:
            raise LookupError("unable to find zone id: %r" % hostid)

        xid     = long(xid)
        args    = obj.get_args()
        changes = []

        rrcache = self.adapter_redis.get_key("nsa.%s.rrcache:%s" % (self.PLUGIN_NAME, hostid))
        nrcache = []

        if not rrcache:
            rrcache = []
        else:
            rrcache = json.loads(rrcache)

        for rrset in args['rrsets']:
            if rrset['name'].endswith(".%s." % hostid):
                name = rrset['name'][:-(len(hostid) + 2)]
            elif rrset['name'] == ("%s." % hostid):
                name = '@'
            else:
                name = rrset['name']

            if rrset['changetype'] == 'REPLACE':
                action = 'upsert'
            else:
                action = rrset['changetype'].lower()

            if rrset['changetype'] == 'DELETE':
                changes.append({'action':      action,
                                'host_name':   name,
                                'record_type': rrset['type']})
                continue

            for record in rrset['records']:
                if record.get('disabled'):
                    continue

                change = {'host_name':   name,
                          'ttl':         rrset.get('ttl') or 0,
                          'record_type': rrset['type']}

                if rrset['type'] == 'MX':
                    (change['data'], change['value']) = record['content'].split(' ', 1)
                elif rrset['type'] == 'NS':
                    change['value'] = record['content'].rstrip('.')
                elif rrset['type'] == 'TXT' \
                   and len(record['content']) > 1 \
                   and record['content'][0] == '"' \
                   and record['content'][-1] == '"':
                    change['value'] = record['content'][1:-1]
                else:
                    change['value'] = record['content']

                nrcache.append(change.copy())

                if not self._is_in_cache(rrcache, change):
                    change['action'] = action
                    changes.append(change)

        if changes:
            r = self.conn.change_records(xid,
                                         changes,
                                         deploy_type = None)
        self.adapter_redis.set_key("nsa.%s.rrcache:%s"
                                   % (self.PLUGIN_NAME, hostid),
                                   json.dumps(nrcache))

    def run(self):
        while True:
            r = None

            try:
                obj  = APIS_SYNC[self.PLUGIN_NAME].qget(True)
                func = "_do_%s" % obj.get_endpoint()
                if not hasattr(self, func):
                    LOG.warning("unknown endpoint function: %r", func)
                    continue

                r    = getattr(self, func)(obj)
            except Exception, e:
                obj.add_error(str(e))
                LOG.exception("%r", e)
            else:
                obj.set_result(r)
            finally:
                obj()


if __name__ != "__main__":
    def _start():
        PLUGINS.register(NSAProxyCdnsPlugin())
    _start()
