# -*- coding: utf-8 -*-
"""r53 plugin"""

__author__  = "Adrien DELLE CAVE"
__license__ = """
    Copyright (C) 2018  fjord-technologies

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA..
"""

import boto3
import logging
import os

from dwho.adapters.redis import DWhoAdapterRedis
from dwho.classes.plugins import PLUGINS
from nsaproxy.classes.apis import NSAProxyApiBase, NSAProxyApiSync, APIS_SYNC
from sonicprobe import helpers


LOG = logging.getLogger('nsaproxy.plugins.r53')

#logging.getLogger('requests').setLevel(logging.WARNING)


class NSAProxyR53Plugin(NSAProxyApiBase):
    PLUGIN_NAME = 'r53'

    def safe_init(self):
        self.conn           = None

        if self.PLUGIN_NAME not in self.config['plugins']:
            return

        self.adapter_redis  = DWhoAdapterRedis(self.config, prefix = 'nsaproxy')

        if self.config['plugins']['r53'].get('credentials'):
            cred = helpers.load_yaml_file(self.config['plugins']['r53']['credentials'])
            if not cred:
                raise ValueError("unable to read credentials")

            for k, v in cred['route53'].iteritems():
                os.environ[k.upper()] = v

        self.conn = boto3.client('route53')

        APIS_SYNC.register(NSAProxyApiSync(self.PLUGIN_NAME))

    def at_start(self):
        if self.PLUGIN_NAME in APIS_SYNC:
            self.start()

    def _do_create_hosted_zone(self, obj):
        args = obj.get_args()
        res = self.conn.list_hosted_zones_by_name(DNSName  = args['name'],
                                                  MaxItems = '1')
        if res and res['HostedZones']:
            self.adapter_redis.set_key("nsa.r53.hzone:%s" % args['name'].rstrip('.'), res['HostedZones'][0]['Id'])
        else:
            r = self.conn.create_hosted_zone(Name            = args['name'],
                                             CallerReference = obj.get_uid())
            self.adapter_redis.set_key("nsa.r53.hzone:%s" % args['name'].rstrip('.'), r['HostedZone']['Id'])

    def _do_delete_hosted_zone(self, obj):
        params = obj.get_params()
        xid    = self.adapter_redis.get_key("nsa.r53.hzone:%s" % params['id'])
        if not xid:
            raise LookupError("unable to find zone id: %r" % params['id'])

        self.conn.delete_hosted_zone(Id = xid)

        self.adapter_redis.del_key("nsa.r53.hzone:%s" % params['id'])

    def _do_change_rrsets(self, obj):
        params  = obj.get_params()
        xid     = self.adapter_redis.get_key("nsa.r53.hzone:%s" % params['id'])
        if not xid:
            res = self.conn.list_hosted_zones_by_name(DNSName  = params['id'],
                                                      MaxItems = '1')
            if not res or not res['HostedZones']:
                raise LookupError("unable to find zone id: %r" % params['id'])
            xid = res['HostedZones'][0]['Id']
            self.adapter_redis.set_key("nsa.r53.hzone:%s" % params['id'], xid)

        args    = obj.get_args()
        changes = {'Comment': '',
                   'Changes': []}

        for rrset in args['rrsets']:
            change = {'ResourceRecordSet': {'Name': rrset['name'],
                                            'TTL':  rrset.get('ttl') or 0,
                                            'Type': rrset['type'],
                                            'ResourceRecords': []}}

            if rrset['changetype'] == 'REPLACE':
                change['Action'] = 'UPSERT'
            else:
                change['Action'] = rrset['changetype']

            if rrset['changetype'] == 'DELETE':
                res = self.conn.list_resource_record_sets(HostedZoneId    = xid,
                                                          StartRecordName = rrset['name'],
                                                          StartRecordType = rrset['type'],
                                                          MaxItems        = '1')
                if not res or not res['ResourceRecordSets']:
                    continue

                change['ResourceRecordSet']['ResourceRecords'] = res['ResourceRecordSets'][0]['ResourceRecords']
                change['ResourceRecordSet']['TTL'] = res['ResourceRecordSets'][0]['TTL']
            else:
                for record in rrset['records']:
                    if not record.get('disabled'):
                        change['ResourceRecordSet']['ResourceRecords'].append({'Value': record['content']})

            if not change['ResourceRecordSet']['ResourceRecords']:
                continue

            changes['Changes'].append(change)

        if changes['Changes']:
            r = self.conn.change_resource_record_sets(HostedZoneId = xid,
                                                      ChangeBatch  = changes)

    def run(self):
        while True:
            r = None

            try:
                obj  = APIS_SYNC[self.PLUGIN_NAME].qget(True)
                func = "_do_%s" % obj.get_endpoint()
                if not hasattr(self, func):
                    LOG.warning("unknown endpoint function: %r", func)
                    continue

                r    = getattr(self, func)(obj)
            except Exception, e:
                obj.add_error(str(e))
                LOG.exception("%r", e)
            else:
                obj.set_result(r)
            finally:
                obj()


if __name__ != "__main__":
    def _start():
        PLUGINS.register(NSAProxyR53Plugin())
    _start()
